# -*- coding: utf-8 -*-

from __future__ import (print_function, unicode_literals, division, absolute_import)

from rapidconnect.settings import DEFAULT_BASE_ENDPOINT
from rapidconnect.request  import Request
from rapidconnect.listen   import Listen

class RapidConnect(object):

    def __init__(self, project, token, base_endpoint=DEFAULT_BASE_ENDPOINT):
        self.project = project
        self.token = token
        self.base_endpoint = base_endpoint

    def call(self, package, block, params):
        request = Request(project=self.project,
                          token=self.token,
                          package=package,
                          block=block,
                          params=params,
                          base_endpoint=self.base_endpoint)

        return request.call

    def listen(self, package, event, params,
               on_join=None, on_message=None, on_close=None, on_error=None):
        event = Listen(self.project, self.token, package, event, params, on_join, on_message, on_close, on_error)
        event.listen()



# -*- coding: utf-8 -*-
from __future__ import (print_function, unicode_literals, division, absolute_import)
import requests
import ast

class Request():
    def __init__(self, project, token, package, block, params, base_endpoint):
        self.project = project
        self.token = token
        self.package = package
        self.block = block
        self.params = params
        self.base_endpoint = base_endpoint

    @property
    def urlBuilder(self):
        return self.base_endpoint + self.package + "/" + self.block

    @property
    def call(self):
        response = requests.post(self.urlBuilder,
                                 auth=(self.project, self.token),
                                 headers={'user-agent': 'RapidAPIConnect_Python'},
                                 data=self.params).text

        objectResponse = ast.literal_eval(response)

        if 'payload' in objectResponse:
            return objectResponse['payload']
        else:
            return objectResponse


import requests
import websocket
import json
import threading

class Listen():
    def __init__(self, project, token, package, event, params,
               on_join=None, on_message=None, on_close=None, on_error=None):
        self.project = project
        self.token = token
        self.on_join = on_join
        self.on_message = on_message
        self.on_close = on_close
        self.on_error = on_error
        self.webhook_params = params
        self.user_id = "%s.%s_%s:%s" % (package, event, self.project, self.token)

    def tokenUrl(self, user_id):
        return "https://webhooks.rapidapi.com/api/get_token?user_id=%s" % (user_id)

    def socketUrl(self, token):
        return "wss://webhooks.rapidapi.com/socket/websocket?token=%s" % (token)

    def on_open(self, ws):
        users_socket = "users_socket:%s" % (self.user_id)
        connect = {'topic': users_socket, 'event': 'phx_join', 'ref': '1', 'payload': self.webhook_params}
        self.ws.send(json.dumps(connect))

    def on_msg_received(self, ws, message):
        decoded = json.loads(message)
        if decoded["event"] == "phx_reply" and decoded["payload"]["status"] == "ok" and self.on_join:
            self.on_join()
            return
        if decoded["event"] == "new_msg" and decoded.get("payload").get("token") == None:
            self.on_error(decoded["payload"]["body"])
        elif decoded["event"] == "new_msg" and decoded["payload"]["token"] == self.socket_token and self.on_message:
            self.on_message(decoded["payload"]["body"])
        elif self.on_error:
            self.on_error(decoded["payload"]["body"])
        return

    def send_heartbeat(self):
        heartbeat = {"topic": "phoenix", "event": "heartbeat", "ref": "1", "payload": {}}
        self.ws.send(json.dumps(heartbeat))

    def on_closed(self, ws):
        self.on_close()
        
    def listen(self):
        response = requests.get(self.tokenUrl(self.user_id), auth=(self.project, self.token))
        decoded = response.json()
        self.socket_token = decoded["token"]
        self.ws = websocket.WebSocketApp(self.socketUrl(self.socket_token),
                                         on_open = self.on_open,
                                         on_error = self.on_closed,
                                         on_message = self.on_msg_received,
                                         on_close = self.on_closed,
                                         on_ping=self.send_heartbeat)
        wst = threading.Thread(target=self.ws.run_forever, kwargs={'ping_interval': 30})
        wst.start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
        name='RapidConnect',
        version='0.0.4',
        author='Andrey Bukati',
        author_email='andrey@rapidapi.com',
        description='Official Python client for RapidAPI Connect',
        url='https://github.com/RapidSoftwareSolutions/RapidAPIConnect_Python',
        keywords=['rapidapi', 'api', 'python'],
        classifiers=[
            'Development Status :: 4 - Beta',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 3',
        ],
        package_dir={'': '.'},
        packages=find_packages('.'),
        install_requires=[
            'requests>=2.8.1',
            'six>=1.10.0',
            'websocket-client>=0.40.0'
        ],
)
